==> eg_particle_interactions-v0.8.1-fabric-mc1.21.8-sources.jar: games/enchanted/eg_particle_interactions/common/mixin/entity/AbstractMinecartMixin.java <==
package games.enchanted.eg_particle_interactions.common.mixin.entity;

import games.enchanted.eg_particle_interactions.common.particle_spawning.SpawnParticles;
import net.minecraft.class_1299;
import net.minecraft.class_1688;
import net.minecraft.class_1937;
import net.minecraft.class_2241;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2680;
import net.minecraft.class_3481;
import net.minecraft.class_3486;
import net.minecraft.class_8836;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(class_1688.class)
public abstract class AbstractMinecartMixin extends class_8836 {
    @Shadow public abstract boolean method_52172();
    @Shadow public abstract @NotNull class_2350 method_5755();
    @Shadow public abstract class_2680 getDisplayBlockState();

    public AbstractMinecartMixin(class_1299<?> entityType, class_1937 level) {
        super(entityType, level);
    }

    @Unique
    private boolean block_place_particle$shouldSpawnSparks() {
        class_2338 blockPos = class_2338.method_49637(this.method_23317(), this.method_23318(),this.method_23321());
        class_2680 blockState = this.method_37908().method_8320(blockPos);
        return !blockState.method_26227().method_15767(class_3486.field_15517);
    }

    @Unique
    private double block_place_particle$maxSpeed() {
        return this.method_5799() ? 0.2 : 0.4;
    }

    @Inject(
        at = @At("HEAD"),
        method = "tick"
    )
    protected void spawnSparksWhileMovingOnRails(CallbackInfo ci) {
        if (block_place_particle$shouldSpawnSparks() && this.method_37908().field_9236) {
            float horizontalRot = this.method_36454();
            float verticalRot = this.method_36455();

            class_2338 blockPos = class_2338.method_49637(this.method_23317(), this.method_23318(),this.method_23321());
            class_2680 blockState = this.method_37908().method_8320(blockPos);
            boolean hasBlock = !this.getDisplayBlockState().method_26164(class_3481.field_51989);

            SpawnParticles.spawnSparksAtMinecartWheels(this.method_23317(), this.method_23318(),this.method_23321(), horizontalRot, verticalRot, class_2241.method_9476(blockState), !this.method_5685().isEmpty(), hasBlock, this.method_18798(), block_place_particle$maxSpeed(), this.method_37908());
        }
    }
}

==> eg_particle_interactions-v0.8.4-fabric-mc1.21.8-sources.jar: games/enchanted/eg_particle_interactions/common/mixin/entity/AbstractMinecartMixin.java <==
package games.enchanted.eg_particle_interactions.common.mixin.entity;

import games.enchanted.eg_particle_interactions.common.particle_spawning.SpawnParticles;
import net.minecraft.class_1299;
import net.minecraft.class_1688;
import net.minecraft.class_1937;
import net.minecraft.class_2241;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2680;
import net.minecraft.class_3481;
import net.minecraft.class_3486;
import net.minecraft.class_8836;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(class_1688.class)
public abstract class AbstractMinecartMixin extends class_8836 {
    @Shadow public abstract boolean method_52172();
    @Shadow public abstract @NotNull class_2350 method_5755();
    @Shadow public abstract class_2680 getDisplayBlockState();

    public AbstractMinecartMixin(class_1299<?> entityType, class_1937 level) {
        super(entityType, level);
    }

    @Unique
    private boolean block_place_particle$shouldSpawnSparks() {
        class_2338 blockPos = class_2338.method_49637(this.method_23317(), this.method_23318(),this.method_23321());
        class_2680 blockState = this.method_37908().method_8320(blockPos);
        return !blockState.method_26227().method_15767(class_3486.field_15517);
    }

    @Unique
    private double block_place_particle$maxSpeed() {
        return this.method_5799() ? 0.2 : 0.4;
    }

    @Inject(
        at = @At("HEAD"),
        method = "tick"
    )
    protected void spawnSparksWhileMovingOnRails(CallbackInfo ci) {
        if (block_place_particle$shouldSpawnSparks() && this.method_37908().method_8608()) {
            float horizontalRot = this.method_36454();
            float verticalRot = this.method_36455();

            class_2338 blockPos = class_2338.method_49637(this.method_23317(), this.method_23318(),this.method_23321());
            class_2680 blockState = this.method_37908().method_8320(blockPos);
            boolean hasBlock = !this.getDisplayBlockState().method_26164(class_3481.field_51989);

            SpawnParticles.spawnSparksAtMinecartWheels(this.method_23317(), this.method_23318(),this.method_23321(), horizontalRot, verticalRot, class_2241.method_9476(blockState), !this.method_5685().isEmpty(), hasBlock, this.method_18798(), block_place_particle$maxSpeed(), this.method_37908());
        }
    }
}
